!  DLL Dipolar2.f90 
!
subroutine DLLDipolar2 (kAB, i, j, lvector, gvector, eigenvectorsreal, eigenvectorscmpx, intensities)

  ! This subroutine calculates transition probabilities (perpendicular and parallel mode)
  ! for any transition within the 4-level system of 2 interacting S=1/2 systems
  ! WRHagen 2019 11 07
  !
    IMPLICIT NONE
  !DEC$ ATTRIBUTES DLLEXPORT::DLLDipolar2
  !DEC$ ATTRIBUTES STDCALL::DLLDipolar2

  ! Declare calling parameters
    INTEGER*4   kAB, i, j
    REAL*8      lvector(3), gvector(3)
    REAL*8      eigenvectorsreal(4,4), eigenvectorscmpx(4,4)
    REAL*8      intensities(2)
    
  ! Declare local parameters
    COMPLEX*8   cm(4,4), smin, spls, sz, sy, sx
    REAL*8      xl, yl, zl, gx, gy, gz, perp, parl

 ! Body of DLLDipolar2
    xl =    lvector(1)
    yl =    lvector(2)
    zl =    lvector(3)
    gx =    gvector(1)
    gy =    gvector(2)
    gz =    gvector(3)

    FORALL (i=1:4, j=1:4)
        cm(i,j) = CMPLX(eigenvectorsreal(i,j), eigenvectorscmpx(i,j))
    END FORALL

    IF (kAB .EQ. 0) THEN
        spls =  CONJG(cm(1,i))*cm(3,j) + CONJG(cm(2,i))*cm(4,j)
        smin =  CONJG(cm(3,i))*cm(1,j) + CONJG(cm(4,i))*cm(2,j)
        sz =    CONJG(cm(1,i))*cm(1,j) + CONJG(cm(2,i))*cm(2,j) - CONJG(cm(3,i))*cm(3,j) - CONJG(cm(4,i))*cm(4,j)
    ELSE    
        spls =  CONJG(cm(1,i))*cm(2,j) + CONJG(cm(3,i))*cm(4,j)
        smin =  CONJG(cm(2,i))*cm(1,j) + CONJG(cm(4,i))*cm(3,j)
        sz =    CONJG(cm(1,i))*cm(1,j) - CONJG(cm(2,i))*cm(2,j) + CONJG(cm(3,i))*cm(3,j) - CONJG(cm(4,i))*cm(4,j) 
    END IF
    sz =    sz / 2.
    
    sx =    (spls + smin) / 2.
    sy =    (spls - smin) / 2. / (0., 1.)
    
    sx =    sx * gx
    sy =    sy * gy
    sz =    Sz * gz
    
    CALL intens (sx, sy, sz, xl, yl, zl, perp, parl)
    
    intensities(1) = perp
    intensities(2) = parl
    
    RETURN

end subroutine DLLDipolar2
