!  DLL DipolarPart.f90 
!
subroutine DLLDipolarpART (ga,gb,rvector,rdistance,zmreal,zmimag)

  ! DLLDipolarPart fills up the Dipolar PART of the energy matrix for a system with two S=1/2 spins 

  ! History:    DLLDipolarNEW   2022 05 25

  ! (C) WRHagen 2023 05 04
  !
IMPLICIT NONE
  !DEC$ ATTRIBUTES DLLEXPORT::DLLDipolarPart
  !DEC$ ATTRIBUTES STDCALL::DLLDipolarPart

  ! Declare calling parameters
    REAL*8, INTENT(IN) ::  ga(3),gb(3,3),rvector(3),rdistance
    REAL*8, INTENT(OUT) :: zmreal(4,4),zmimag(4,4)
    
  ! Declare local parameters
    INTEGER i,j
    REAL*8  beta
    REAL*8  dreal1(4,4),dimag1(4,4),dreal2(4,4),dimag2(4,4)
    REAL*8  ax,ay,az,bx,by,bz,SIfactor

  ! Body of DLLDipolarNEW
    beta = 0.46689088 / 10000.  !in reciprocal cm per gauss
    
    FORALL (i=1:4,j=1:4)
        dreal1(i,j) = 0.0
        dimag1(i,j) = 0.0
        dreal2(i,j) = 0.0
        dimag2(i,j) = 0.0
        zmreal(i,j) = 0.0
        zmimag(i,j) = 0.0
    END FORALL
    
  ! First dipolar term
    dreal1(1,1) =  ga(3) * gb(3,3)
    dreal1(2,3) =  ga(1)*gb(1,1) + ga(2)*gb(2,2)
    dreal1(2,2) = -dreal1(1,1)
    dreal1(3,3) = -dreal1(1,1)
    dreal1(4,4) =  dreal1(1,1)
             dreal1(3,2) = dreal1(2,3)
    dreal1(1,2) =  ga(3) * gb(3,1)
    dreal1(1,3) =  ga(1) * gb(1,3)
    dreal1(1,4) =  ga(1)*gb(1,1) - ga(2)*gb(2,2)
    dreal1(2,4) = -dreal1(1,3)      !- (cf Dipolar1 on 20220506)
    dreal1(3,4) = -dreal1(1,2)      !-
            dreal1(2,1) = dreal1(1,2)
            dreal1(3,1) = dreal1(1,3)
            dreal1(4,1) = dreal1(1,4)
            dreal1(4,2) = dreal1(2,4)
            dreal1(4,3) = dreal1(3,4)
    dreal1 = dreal1 / 4.
    
    dimag1(2,3) =  ga(1)*gb(1,2) - ga(2)*gb(2,1)
            dimag1(3,2) = -dimag1(2,3)
    dimag1(1,2) = -ga(3) * gb(3,2)
    dimag1(1,3) = -ga(2) * gb(2,3)
    dimag1(1,4) = -ga(1)*gb(1,2) - ga(2)*gb(2,1)
    dimag1(2,4) = -dimag1(1,3)      !-
    dimag1(3,4) = -dimag1(1,2)      !-
            dimag1(2,1) = -dimag1(1,2)
            dimag1(3,1) = -dimag1(1,3)
            dimag1(4,1) = -dimag1(1,4)
            dimag1(4,2) = -dimag1(2,4)
            dimag1(4,3) = -dimag1(3,4)
    dimag1 = dimag1 / 4.
    
  ! Second dipolar term
    ax = rvector(1) * ga(1)
    ay = rvector(2) * ga(2)
    az = rvector(3) * ga(3)
    bx = rvector(1)*gb(1,1) + rvector(2)*gb(2,1) + rvector(3)*gb(3,1)
    by = rvector(1)*gb(1,2) + rvector(2)*gb(2,2) + rvector(3)*gb(3,2)
    bz = rvector(1)*gb(1,3) + rvector(2)*gb(2,3) + rvector(3)*gb(3,3)
    
    dreal2(1,1) =  az * bz
    dreal2(2,2) = -dreal2(1,1)
    dreal2(3,3) = -dreal2(1,1)
    dreal2(4,4) =  dreal2(1,1)
    dreal2(2,3) =  ax*bx + ay*by
             dreal2(3,2) = dreal2(2,3)
    dreal2(1,2) =  az * bx
    dreal2(1,3) =  ax * bz
    dreal2(1,4) =  ax*bx - ay*by
    dreal2(2,4) = -dreal2(1,3)      !-
    dreal2(3,4) = -dreal2(1,2)      !-
            dreal2(2,1) = dreal2(1,2)
            dreal2(3,1) = dreal2(1,3)
            dreal2(4,1) = dreal2(1,4) 
            dreal2(4,2) = dreal2(2,4)
            dreal2(4,3) = dreal2(3,4)
    dreal2 = dreal2 / 4.
    
    dimag2(2,3) =  ax*by - ay*bx
             dimag2(3,2) = -dimag2(2,3)
    dimag2(1,2) = -az * by
    dimag2(1,3) = -ay * bz
    dimag2(1,4) = -ax*by - ay*bx
    dimag2(2,4) = -dimag2(1,3)      !-
    dimag2(3,4) = -dimag2(1,2)      !-
            dimag2(2,1) = -dimag2(1,2)
            dimag2(3,1) = -dimag2(1,3)
            dimag2(4,1) = -dimag2(1,4) 
            dimag2(4,2) = -dimag2(2,4)
            dimag2(4,3) = -dimag2(3,4)
    dimag2 = dimag2 / 4.
    
    dreal2 = -3.0 * dreal2 / (rdistance*rdistance)
    dimag2 = -3.0 * dimag2 / (rdistance*rdistance)
    
  ! Next lines taken from DLLDIP8CUBEN:
  ! Note that in the point-dipole equation (book 13, page 47):
  ! the first beta is in wavenumbers/gauss since hv is calculated in wavenumbers;
  ! and the second beta is in SI units: joule/tesla.
  ! The pre-factor of the dipolar term is mu-zero x beta-squared / 4 pi x r^3,
  ! wherin beta-squared is first beta (wavenumbers/gauss) times second beta (joule/tesla).
  ! The pre-factor is first-beta x SIfactor.
    
  ! SIfactor is (mu-nul/4pi) * (beta/r**3) with ALL quantities in SI units
  ! and for F in Hertz (not MHz) and rdistance from cm (not m) to Angstrom 
    SIfactor = 9.274009994 * 1000.   
    zmreal = zmreal + beta * SIfactor * (dreal1 + dreal2) / (rdistance*rdistance*rdistance)
    zmimag = zmimag + beta * SIfactor * (dimag1 + dimag2) / (rdistance*rdistance*rdistance)
    
end subroutine DLLDipolarPart
