!  DLLZeemanPart.f90 

subroutine DLLZeemanPart (lvector,ga,breala,bimaga)

  ! DLLDipolarNEW fills up the UPPER TRIANGULAR PART of the energy matrix for a system with two S=1/2 spins 
  ! subject to dipolar interaction.
  ! The math and format/algoritm is according to the NEW procedures developed in BOOK 23, pp 13-23.

  ! History:    DLLDipolarNew 2022 05 25

  ! (C) WRHagen 2023 05 04

  !
IMPLICIT NONE
  !DEC$ ATTRIBUTES DLLEXPORT::DLLZeemanPart
  !DEC$ ATTRIBUTES STDCALL::DLLZeemanPart

  ! Declare calling parameters
    REAL*8, INTENT(IN) ::  lvector(3),ga(3)
    REAL*8, INTENT(OUT) :: breala(4,4),bimaga(4,4)
    
  ! Declare local parameters
    INTEGER i,j
    REAL*8  beta,lx,ly,lz,gxa,gya,gza

  ! Body of DLLDipolarNEW
    beta = 0.46689088 / 10000.  !in reciprocal cm per gauss
    lx = lvector(1)
    ly = lvector(2)
    lz = lvector(3)
    
    FORALL (i=1:4,j=1:4)
        breala(i,j) = 0.0
        bimaga(i,j) = 0.0
    END FORALL
    
  ! Zeeman terms for B = 1.000
    gxa = beta * ga(1) * lx / 2.
    gya = beta * ga(2) * ly / 2.
    gza = beta * ga(3) * lz / 2.

    breala(1,1) = gza
    breala(1,3) = gxa
    breala(2,2) = breala(1,1)
    breala(2,4) = breala(1,3)
    breala(3,3) = -breala(1,1)
    breala(4,4) = -breala(1,1)
  ! All next 8-position indented lines are to fill up the lower triangular part of the matrices.
  ! This is not really necessary, since LabVIEW's subVI 'Eigenvalues and Vectors' apparently takes the
  ! upper (as defined in FORTRAN) triangular part, so it must be based on a variant of the EISPACK routines.
            breala(3,1) = breala(1,3)
            breala(4,2) = breala(2,4)
    
    bimaga(1,3) = -gya
    bimaga(2,4) = bimaga(1,3)
            bimaga(3,1) = -bimaga(1,3)
            bimaga(4,2) = -bimaga(2,4)
    
end subroutine DLLZeemanPart
